package com.oqpo.api.service;

import com.oqpo.api.config.JwtTokenProvider;
import com.oqpo.api.constant.ApiConstants;
import com.oqpo.api.entity.oper.MemberEntity;
import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.exception.GlobalException;
import com.oqpo.api.mapper.MemberMapper;
import com.oqpo.api.mapper.oper.SaleMgrMgnMapper;
import com.oqpo.api.mapper.oper.UserMngMapper;
import com.oqpo.api.mapper.stinfo.BrandMapper;
import com.oqpo.api.service.oper.UserMngService;
import com.oqpo.api.util.StringUtil;
import com.oqpo.api.web.dto.response.oper.ntice.NticeTargetResponse;
import com.oqpo.api.web.dto.response.oper.usermng.UserInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqRejectTargetResponse;
import com.oqpo.api.web.dto.response.session.FindIdResponse;
import com.oqpo.api.web.dto.response.session.FindPwResponse;
import com.oqpo.api.web.dto.response.session.LogInBrandListResponse;
import com.oqpo.api.web.dto.response.token.JwtResponse;
import com.oqpo.api.web.dto.response.token.TokenResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class SigninService implements UserDetailsService {
    @Autowired
    private MemberMapper memberMapper;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private JwtTokenProvider jwtTokenProvider;

    @Autowired
    private UserMngService userMngService ;

    @Autowired
    private SaleMgrMgnMapper saleMgrMgnMapper;

    @Autowired
    private BrandMapper brandMapper;


    public JwtResponse signin(String userId, String password) throws Exception {
        MemberEntity memberEntity = null;
        String connKey = "";
        try {
            UsernamePasswordAuthenticationToken token = new UsernamePasswordAuthenticationToken(userId, password);
            Authentication authentication = authenticationManager.authenticate(token);
            SecurityContextHolder.getContext().setAuthentication(authentication);
            memberEntity = (MemberEntity) authentication.getPrincipal();
            //System.out.println("==getUserNm=====================> " + memberEntity.getUserNm());
            connKey = jwtTokenProvider.createToken (
                    memberEntity.getUserId(),
                    memberEntity.getUserNm(),
                    memberEntity.getSystemAdminYn(),
                    memberEntity.getAuthTpCd(),
                    true,
                    memberEntity.getAfflShopId(),
                    memberEntity.getBrandId(),
                    memberEntity.getAuthTpNm());
            log.debug("connKey==>>>{}", connKey);


            // 단말 정보 업데이트


            //로그인 이력 남기기
            userMngService.updateLoginDt(memberEntity.getUserId());

        } catch (Exception e) {
            //e.getStackTrace();
            e.printStackTrace();
            throw new GlobalException(SystemMessageCode.SIGNIN_FAIL);
        }
        return JwtResponse.toDTO(connKey);
    }


    public JwtResponse signinApp(String brandId, String userId, String password, String deviId, String deviTpCd, String deviOsVer, String appReg, String appVer) throws Exception {
        MemberEntity memberEntity = null;
        String connKey = "";
        try {

            System.out.println("====================================================") ;

            UsernamePasswordAuthenticationToken token = new UsernamePasswordAuthenticationToken(userId, password);
            Authentication authentication = authenticationManager.authenticate(token);
            SecurityContextHolder.getContext().setAuthentication(authentication);
            memberEntity = (MemberEntity) authentication.getPrincipal();

            // 선택된 브랜드 소속 로그인자 인지 확인 필요 ( 10  무시, 20 영업담당자 관리 브랜드그룹체크, 30 브랜드 동일여부 체크, 40 브랜드 소속 체크 50 브랜드소속, 60 브랜드소속)
            int existCnt = 1 ;
            if ("20".equals(memberEntity.getAuthTpCd())) {
                //20 영업담당자 관리 브랜드그룹체크
                    existCnt = saleMgrMgnMapper.selectSaleMgrBrandExist(brandId, userId);
            } else if ("30".equals(memberEntity.getAuthTpCd())) {
                // 30 브랜드 동일여부 체크
                    existCnt = brandMapper.selectAfflBrandExist(brandId, memberEntity.getAfflShopId());
            }  else if ("40".equals(memberEntity.getAuthTpCd())  || "50".equals(memberEntity.getAuthTpCd()) || "60".equals(memberEntity.getAuthTpCd())) {
                    if (!brandId.equals(memberEntity.getBrandId())) {
                        existCnt = 0 ;
                    }
            }

            System.out.println("==existCnt=====================> " + existCnt);


            if (existCnt == 0 ) {
                throw new GlobalException(SystemMessageCode.ERR_NONEXISTBRAND);
            }


            System.out.println("==getUserNm=====================> " + memberEntity.getUserNm());
            connKey = jwtTokenProvider.createToken (
                    memberEntity.getUserId(),
                    memberEntity.getUserNm(),
                    memberEntity.getSystemAdminYn(),
                    memberEntity.getAuthTpCd(),
                    true,
                    memberEntity.getAfflShopId(),
                    brandId,
                    memberEntity.getAuthTpNm());
            log.debug("connKey==>>>{}", connKey);

            //로그인 이력 남기기
            System.out.println("deviId====================================================>" + deviId) ;

            if (deviId == null || "".toString().equals(deviId)) {
                System.out.println("deviId====================================================>isnull") ;
                userMngService.updateLoginDt(memberEntity.getUserId());
            } else {
                System.out.println("deviId====================================================>exist") ;
                userMngService.updateAppLoginDt(memberEntity.getUserId(), deviId,  deviTpCd,  deviOsVer,  appReg,  appVer);
            }


        } catch (GlobalException e) {
            System.out.println("==GlobalException=====================> ");
            e.printStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            System.out.println("==Exception=====================> ");
            e.printStackTrace();
            throw new GlobalException(SystemMessageCode.SIGNIN_FAIL);
        }
        return JwtResponse.toDTO(connKey);
    }


    public void logout(HttpServletRequest request, HttpServletResponse response) throws Exception {
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        if (auth != null) {
            new SecurityContextLogoutHandler().logout(request, response, auth);
        }
    }

    public UserDetails getMember(String userId) {
        MemberEntity entity = memberMapper.selectMemberById(userId);
        entity.setAuthorities(getAuthorities());
        return entity;
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        return getMember(username);
    }

    public Collection<GrantedAuthority> getAuthorities() {
        List<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();
        authorities.add(new SimpleGrantedAuthority("ROLE_SRM_MEMBER"));
        return authorities;
    }

    public PasswordEncoder passwordEncoder() {
        return this.passwordEncoder;
    }

    public FindIdResponse selectMemberFindId(String userNm, String email, String telNo) {
        MemberEntity entity = memberMapper.selectMemberFindId(userNm, email, telNo);

        if (entity == null) {
            throw new GlobalException(SystemMessageCode.ERR_FIND_ID);
        }

        return FindIdResponse.toDTO(entity);
    }

    public FindPwResponse selectMemberFindPw(String userId, String email, String telNo) throws Exception {
        String newpw = "";
        MemberEntity entity = new MemberEntity() ;
        int idCnt = memberMapper.selectMemberFindPwCnt(userId, email, telNo);

        if (idCnt == 0) {
            throw new GlobalException(SystemMessageCode.ERR_FIND_PW);
        } else {
            newpw = StringUtil.makeSecureString(6) ;
            entity.setUserId(userId);
            entity.setPassword(newpw);

            //패스워드 변경처리함
            userMngService.updateChgUserPw(userId, newpw);
        }

        return FindPwResponse.toDTO(entity);
    }

    //selectMemberBrandList
    public LogInBrandListResponse selectMemberBrandList(String bsnsRegNo) throws Exception {

        return LogInBrandListResponse.toDTO(memberMapper.selectMemberBrandList(bsnsRegNo));
    }

}
