package com.oqpo.api.service.pomng;


import com.oqpo.api.entity.inoutmng.DeliDtlMgntEntity;
import com.oqpo.api.entity.inoutmng.DeliInfoMgntEntity;
import com.oqpo.api.entity.inoutmng.ShmtOdrBaseEntity;
import com.oqpo.api.entity.inoutmng.ShmtOdrDtlEntity;
import com.oqpo.api.entity.loanmng.LoanMgntBaseInfoEntity;
import com.oqpo.api.entity.oper.UserMngEntity;
import com.oqpo.api.entity.pomng.PchOdrBaseInfoEntity;
import com.oqpo.api.entity.pomng.PchOdrDtlPtclEntity;
import com.oqpo.api.entity.pomng.PchReqBaseInfoEntity;
import com.oqpo.api.entity.pomng.PchReqDtlPtclEntity;
import com.oqpo.api.entity.rtnmng.RtnReqBaseInfoEntity;
import com.oqpo.api.entity.stockmng.StckMgntBaseInfoEntity;
import com.oqpo.api.entity.stockmng.StckMgntDtlHistEntity;
import com.oqpo.api.entity.stockmng.WhsMgntBaseInfoEntity;
import com.oqpo.api.enums.ChgDvsn;
import com.oqpo.api.enums.InOutCd;
import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.exception.GlobalException;
import com.oqpo.api.mapper.inoutmng.ShmtOdrMapper;
import com.oqpo.api.mapper.loanmng.LoanStateMapper;
import com.oqpo.api.mapper.oper.UserMngMapper;
import com.oqpo.api.mapper.pomng.PchOrdMapper;
import com.oqpo.api.mapper.pomng.PchReqMapper;
import com.oqpo.api.mapper.stinfo.StoreBaseInfoMapper;
import com.oqpo.api.mapper.stockmng.StockMngMapper;
import com.oqpo.api.mapper.stockmng.WhsMngMapper;
import com.oqpo.api.service.CommonService;
import com.oqpo.api.web.dto.request.GridRequest;
import com.oqpo.api.web.dto.request.pomng.pchOrd.*;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.pomng.pchOrd.*;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqInfoResponse;
import com.oqpo.api.web.dto.response.pomng.pchReq.PchReqSearchListResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class PchOrdService extends CommonService {
    @Autowired
    private PchOrdMapper pchOrdMapper;

    @Autowired
    private UserMngMapper userMngMapper;

    @Autowired
    private StockMngMapper stockMngMapper;

    @Autowired
    private PchReqMapper pchReqMapper;

    @Autowired
    private StoreBaseInfoMapper storeBaseInfoMapper;

    @Autowired
    private LoanStateMapper loanStateMapper;

    @Autowired
    private PchReqService pchReqService;

    @Autowired
    private WhsMngMapper whsMngMapper;

    @Autowired
    private ShmtOdrMapper shmtOdrMapper;



    /* 구매요청건 발주 가능 여부 체크 */
    public String  chkPchReqUseOrd(String userId, PchReqOrdInitRequest pchReqOrdInitRequest ) throws Exception {
        String usePassPoYn = "Y";
        // 초기 발주 head 정보 가져오기
        List<PchOdrBaseInfoEntity> headList = null;
        List<PchOdrDtlPtclEntity> itemList = null;

        // 품목 데이타 확인 필요
        List<PchOdrDtlPtclEntity> gridUpdateData = pchReqOrdInitRequest.toCodeEntities(pchReqOrdInitRequest.getGridUpdateData());
        if (gridUpdateData.size() > 0) {
            headList = pchOrdMapper.selectRecPchInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
            itemList = pchOrdMapper.selectRecPchDtlInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
        }

        if (headList.size() > 1) {
            // 서로다른 납품장소로 인한 처리 불가 오류 메세지 출력
            usePassPoYn = "N";
            throw new GlobalException(SystemMessageCode.ERR_NEQWHSID);
        }

        PchOdrBaseInfoEntity headData =  headList.get(0);
        String dlvReqDt = ""; // 납품요청일
        String spplyId = ""; // 공급업체

        if ("DL01".equals(headData.getDlvDvsn())) {
              // 본사 배송은 매장으로의 배송이며, 납품장소는 매장이어야 함
              // 구매요청건이 같은 매장이어야 하며, 납품요청일 동일해야함
            if ("".equals(headData.getStoreId())) {
                usePassPoYn = "N";
                throw new GlobalException(SystemMessageCode.ERR_NONSTORE);
            }

            /* 납품요청일은 동일건 체크 안함 20230119
            for (int j = 0 ; j < itemList.size() ; j++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(j);
                if ("".equals(dlvReqDt)) {
                    dlvReqDt = itemInfo.getDlvReqDt();
                } else {
                    if (!dlvReqDt.equals(itemInfo.getDlvReqDt())) {
                        usePassPoYn = "N";
                        throw new GlobalException(SystemMessageCode.ERR_NEQDLVDT);
                    }
                }
            }
            */
        } else if ("DL02".equals(headData.getDlvDvsn())) {
            // 납품매장으로 - 공급사 배송에서 본사로 배송
            for (int i = 0; i < itemList.size(); i++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(i);
                if ("".equals(spplyId)) {
                    spplyId = itemInfo.getSpplyId();
                } else {
                    if (!spplyId.equals(itemInfo.getSpplyId())) {
                        usePassPoYn = "N";
                        throw new GlobalException(SystemMessageCode.ERR_NEQSPPLY);
                    }
                }
                if ("".equals(spplyId)) {
                    usePassPoYn = "N";
                    throw new GlobalException(SystemMessageCode.ERR_NONSPPLY);
                }

            }
        }

        return usePassPoYn;
    }


    /* 구매요청 발주  초기 정보
    *
    * 1. 구매요청에서 선택된 품목에 대해 발주 대상 품목 가능 여부 - 납품장소가 매장이면  납품요청일 + , 납품장소가 본사창고이면 납품요청일 + 납품공급사 ㅇ여부 체크하고 이상없을시
    *     발주생성화면으로 전환
    * 2. 발주 초기데이타에 대해 한번더 점검한후 발주생성 --
    * */

    public PchOrdInitInfoResponse selectRecPchInitInfo(String userId, PchReqOrdInitRequest pchReqOrdInitRequest ) throws Exception {

        // 로그인 정보 가져오기
        UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);

        // 초기 발주 head 정보 가져오기
        List<PchOdrBaseInfoEntity> headList = null;
        List<PchOdrDtlPtclEntity> itemList = null;

        // 품목 데이타 확인 필요
        List<PchOdrDtlPtclEntity> gridUpdateData = pchReqOrdInitRequest.toCodeEntities(pchReqOrdInitRequest.getGridUpdateData());
        if (gridUpdateData.size() > 0) {
            headList = pchOrdMapper.selectRecPchInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
            itemList = pchOrdMapper.selectRecPchDtlInitInfo(pchReqOrdInitRequest.getBrandId(),gridUpdateData);
        }

        if (headList.size() > 1) {
            // 서로다른 납품장소로 인한 처리 불가 오류 메세지 출력
            throw new GlobalException(SystemMessageCode.ERR_NEQWHSID);
        }

        PchOdrBaseInfoEntity headData =  headList.get(0);
        long podrTotalAmt = 0 ; // 총 발주금액
        String dlvReqDt = ""; // 납품요청일
        String spplyId = ""; // 공급업체
        String spplyNm = ""; //공급업체명


        if ("DL01".equals(headData.getDlvDvsn())) {
            // 본사에서 매장으로 배송처리
            if ("".equals(headData.getStoreId())) {
                throw new GlobalException(SystemMessageCode.ERR_NONSTORE);
            }

            for (int j = 0 ; j < itemList.size() ; j++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(j);
                podrTotalAmt = podrTotalAmt + (itemInfo.getUnitAmt() * itemInfo.getPchReqQty());
                if ("".equals(dlvReqDt)) {
                   // System.out.println(" before==dlvReqDt===================>" + dlvReqDt);
                    dlvReqDt = itemInfo.getDlvReqDt();
                   // System.out.println(" after==dlvReqDt===================>" + dlvReqDt);
                } else {
                    if (dlvReqDt.compareTo(itemInfo.getDlvReqDt()) < 0) {
                        dlvReqDt = itemInfo.getDlvReqDt();
                        //throw new GlobalException(SystemMessageCode.ERR_NEQDLVDT);
                    }
                }
            }
        } else if ("DL02".equals(headData.getDlvDvsn())) {
            // 매장으로 납품 (본사에서 발송할지 공급사에서 발송할지 여부 판단. 기본은 본사에서 발송처리되어지도록 함)
            for (int i = 0 ; i < itemList.size() ;i++) {
                PchOdrDtlPtclEntity itemInfo = itemList.get(i);
                podrTotalAmt = podrTotalAmt + (itemInfo.getUnitAmt() * itemInfo.getPchReqQty());
                if ("".equals(spplyId)) {
                    spplyId = itemInfo.getSpplyId();
                    spplyNm = itemInfo.getSpplyNm();
                } else {
                    if (!spplyId.equals(itemInfo.getSpplyId())) {
                        throw new GlobalException(SystemMessageCode.ERR_NEQSPPLY);
                    }
                }
                if ("".equals(spplyId)) {
                    throw new GlobalException(SystemMessageCode.ERR_NONSPPLY);
                }

                dlvReqDt = itemInfo.getDlvReqDt();
            }
        }
       // System.out.println(" last==dlvReqDt===================>" + dlvReqDt);
        headData.setSpplyId(spplyId);
        headData.setSpplyNm(spplyNm);
        headData.setPodrTotalAmt(podrTotalAmt);
        headData.setDlvReqDt(dlvReqDt);
       // System.out.println(" headData==dlvReqDt===================>" + headData.getDlvReqDt());


        return PchOrdInitInfoResponse.toDTO(uEntity, headData, itemList);
    }

    /*
    발주 신규생성 초기데이타
     */
    public PoOrdInitInfoResponse selectPoInitInfo(String userId, String brandId) throws Exception {

        // 로그인 정보 가져오기
        UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);

        // 초기 발주 head 정보 가져오기
        PchOdrBaseInfoEntity headData = pchOrdMapper.selectPoInitInfo(brandId);

        return PoOrdInitInfoResponse.toDTO(uEntity, headData);
    }

    /*
      발주 정보 저장
     */
    @Transactional
    public void savePchOrdInfo(String userId, SavePchOrdRequest savePchOrdRequest) throws Exception {
        try {

            // 로그인 정보 가져오기
            UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);
            uEntity.setTelNo(uEntity.getTelNo().replaceAll("-",""));

            // 전달받은 데이타와 전송발 데이타 세팅 후 저장
            String pchPodrUnqNo = fnGetDealNo(14,"");
            PchOdrBaseInfoEntity pchOrdEntity = new PchOdrBaseInfoEntity();
            pchOrdEntity.setPchPodrUnqNo(pchPodrUnqNo) ;
            pchOrdEntity.setBrandId(savePchOrdRequest.getBrandId());
            pchOrdEntity.setDlvDvsn(savePchOrdRequest.getDlvDvsn());
            pchOrdEntity.setDlvReqDt(savePchOrdRequest.getDlvReqDt().replaceAll("\\.",""));
            pchOrdEntity.setPodrMgrNm(uEntity.getUserNm());
            pchOrdEntity.setPodrDvsn("OD01");
            pchOrdEntity.setPchOdrStCd("PO20");
            pchOrdEntity.setSpplyId(savePchOrdRequest.getSpplyId());
            pchOrdEntity.setPodrMgrId(uEntity.getUserId());
            pchOrdEntity.setPodrMgrTelNo(uEntity.getTelNo());
            //
            if ("".equals(savePchOrdRequest.getStoreId())) {  // 본사납품장소
                pchOrdEntity.setWhsDvsn("W01");
            } else {  // 매장납품장소
                pchOrdEntity.setWhsDvsn("W02");
            }
            pchOrdEntity.setWhsId(savePchOrdRequest.getWhsId());
            pchOrdEntity.setLocation(savePchOrdRequest.getLocation());
            // 본사납품장소이면 아래 데이타는 null값 넘어옴
            pchOrdEntity.setShmtWhsId(savePchOrdRequest.getShmtWhsId());
            pchOrdEntity.setShmtLocation(savePchOrdRequest.getShmtLocation());
            pchOrdEntity.setStoreId(savePchOrdRequest.getStoreId());

            //수량변동에 따른 금액 변동 합
            long chgPrAmt = 0 ;

            Integer dtlRow = 0 ;
            List<PchOdrDtlPtclEntity> pchOrdDtlList = new ArrayList<PchOdrDtlPtclEntity>();
            List<PchOdrDtlPtclEntity> gridUpdateData = savePchOrdRequest.toCodeEntities(savePchOrdRequest.getGridInsertData());

            // 헤더데이타 체크
            if (gridUpdateData.size() < 1) {
                throw new GlobalException(SystemMessageCode.ERR_NOTITEM);
            }


            //List<PchReqBaseInfoEntity> pchReqList = new ArrayList<PchReqBaseInfoEntity>();
            List<PchReqDtlPtclEntity> pchReqDtlList = new ArrayList<PchReqDtlPtclEntity>();
            if (gridUpdateData.size() > 0) {
                long poTotAmt = 0 ;
                int poItemCnt = 0 ;
                String  diffSpplyYn = "N" ;
                for (int i = 0 ; i < gridUpdateData.size() ;i++ ) {
                    PchOdrDtlPtclEntity poDtlEntity = gridUpdateData.get(i);
                    PchOdrDtlPtclEntity poSaveDtlEntity = new PchOdrDtlPtclEntity();
                    //PchReqBaseInfoEntity pchReqEntity = new PchReqBaseInfoEntity();
                    PchReqDtlPtclEntity pchReqDtlEntity = new PchReqDtlPtclEntity();
                    // 구매요청건에 대한 데이타 가져옴
                    if ("R".equals(savePchOrdRequest.getJobCd())) {   // 구매요청건
                        // 구매요청번호와 항번으로 데이타 가져놈
                        poSaveDtlEntity = pchOrdMapper.selectPoPchDtlSaveInfo(poDtlEntity.getPchReqUnqNo(), poDtlEntity.getPchReqDtlNo());

                        pchReqDtlEntity.setPchReqUnqNo(poDtlEntity.getPchReqUnqNo());
                        pchReqDtlEntity.setPchReqDtlNo(poDtlEntity.getPchReqDtlNo());
                        pchReqDtlEntity.setPchReqDtlStCd("PRD30");
                        pchReqDtlList.add(pchReqDtlEntity);
                        //pchReqEntity.setPchReqUnqNo(poDtlEntity.getPchReqUnqNo());
                        //pchReqEntity.setPchReqStCd("PR30");
                        //pchReqList.add(pchReqEntity);
                    } else if ("P".equals(savePchOrdRequest.getJobCd())) {  // 신규발주건
                        // item id 로 데이타 가져옴  == 단가는 브랜드 단가로 처리함
                        poSaveDtlEntity = pchOrdMapper.selectPoItemDtlSaveInfo(savePchOrdRequest.getBrandId(), poDtlEntity.getItemId(), poDtlEntity.getBrandUnitUnqNo());
                    }

                    if (!pchOrdEntity.getSpplyId().equals(poSaveDtlEntity.getSpplyId())) {
                        diffSpplyYn = "Y";
                    }

                    //수량 변동에 따른 구매사용금액 차감 처리 필요
                   if ("R".equals(savePchOrdRequest.getJobCd()) && poDtlEntity.getPchReqQty() > poDtlEntity.getPodrQty()) {
                       if ("DL02".equals(savePchOrdRequest.getDlvDvsn())) {  //공급사 배송
                           chgPrAmt = chgPrAmt + (poDtlEntity.getPchReqQty() - poDtlEntity.getPodrQty()) * poSaveDtlEntity.getUnitAmt();
                       } else {
                           chgPrAmt = chgPrAmt + (poDtlEntity.getPchReqQty() - poDtlEntity.getPodrQty()) * poSaveDtlEntity.getUnitAmt();
                       }
                    }

                    if ("R".equals(savePchOrdRequest.getJobCd())) {   // 구매요청건
                        if ("DL02".equals(savePchOrdRequest.getDlvDvsn())) {  //공급사 배송
                            poSaveDtlEntity.setUnitAmt(poSaveDtlEntity.getSpplyUnitAmt());
                        }
                    }

                    dtlRow = dtlRow + 10;
                    poSaveDtlEntity.setPchPodrUnqNo(pchPodrUnqNo);
                    String dtlNo = Integer.toString(dtlRow);
                    poSaveDtlEntity.setPchPodrDtlNo(dtlNo);
                    poSaveDtlEntity.setPchReqUnqNo(poDtlEntity.getPchReqUnqNo());
                    poSaveDtlEntity.setPchReqDtlNo(poDtlEntity.getPchReqDtlNo());
                    poSaveDtlEntity.setPodrQty(poDtlEntity.getPodrQty());
                    poSaveDtlEntity.setPodrAmt(Long.valueOf(poDtlEntity.getPodrQty()) * poSaveDtlEntity.getUnitAmt() );
                    poSaveDtlEntity.setCurr("KRW");
                    poSaveDtlEntity.setPodrDtlStCd("POD0");
                    poSaveDtlEntity.setBrandUnitUnqNo(poDtlEntity.getBrandUnitUnqNo());
                    poSaveDtlEntity.setStoreUnitUnqNo(poDtlEntity.getStoreUnitUnqNo());
                    poTotAmt = poTotAmt +  poSaveDtlEntity.getPodrAmt();
                    poItemCnt = poItemCnt + 1;

                    pchOrdDtlList.add(poSaveDtlEntity);
                }

                // 체크로직필요 - w01 납품창고가 본사
                if ("W01".equals(pchOrdEntity.getWhsDvsn())) {
                    // 본사납품장소일경우 - 공급사는 하나여야만한다.
                    if(!"DL02".equals(pchOrdEntity.getDlvDvsn())) {
                        // 본사납품장소이면 배송구분은 공급사이여야 함
                        throw new GlobalException(SystemMessageCode.ERR_DLV001);
                    }
                    // 공급사는 하나여야 함
                    if ("Y".equals(diffSpplyYn)) {
                        // 공급사는 하나여야 합니다.
                        throw new GlobalException(SystemMessageCode.ERR_NEQSPPLY);
                    }
                } else {
                    // 매장납품장소일경우 - 배송구분이 공급사면 - 공급사 하나여야 하고, 본사이면 매장이 하나여야 한다.
                    if ("DL01".equals(pchOrdEntity.getDlvDvsn())) { // 본사배송이면
                        if ("".equals(pchOrdEntity.getStoreId())) {
                            // 매장정보가 없습니다.
                            throw new GlobalException(SystemMessageCode.ERR_NONSTOREID);
                        }

                        //출고창고정보 존재여부 파악
                        if ("".equals(pchOrdEntity.getShmtWhsId()) || "".equals(pchOrdEntity.getShmtLocation())) {
                            throw new GlobalException(SystemMessageCode.ERR_OUTWHS);
                        }

                    } else {  // 공급사배송이면
                        // 공급사는 하나여야 함
                        if ("Y".equals(diffSpplyYn)) {
                            // 공급사는 하나여야 합니다.
                            throw new GlobalException(SystemMessageCode.ERR_NEQSPPLY);
                        }
                    }
                }

                pchOrdEntity.setPodrTotalAmt(poTotAmt);
                pchOrdEntity.setPodrItemQty(poItemCnt);

                pchOrdMapper.insertPchOdrBaseInfo(userId, pchOrdEntity);
                pchOrdMapper.insertPchOdrDtlPtcl(userId, pchOrdDtlList);

                // 구매요청건에 대한 상태에 대한 처리상태 변경
                if ("R".equals(savePchOrdRequest.getJobCd())) {   // 구매요청건
                    pchReqMapper.updateSelPchReqDtlStCd(userId,pchReqDtlList);
                    String pchReqUnqNo = "";
                    List<PchReqBaseInfoEntity> pchReqList = new ArrayList<PchReqBaseInfoEntity>();
                    for (int k = 0; k < pchReqDtlList.size(); k++) {
                        if (!pchReqUnqNo.equals(pchReqDtlList.get(k).getPchReqUnqNo())) {
                            pchReqUnqNo = pchReqDtlList.get(k).getPchReqUnqNo();
                            String pchReqStCd = pchReqService.selectPchReqDtlStatusList(pchReqUnqNo);
                            PchReqBaseInfoEntity pchReqEntity = new PchReqBaseInfoEntity();  // 구매요청상태
                            pchReqEntity.setPchReqUnqNo(pchReqUnqNo);
                            pchReqEntity.setPchReqStCd(pchReqStCd);
                            pchReqList.add(pchReqEntity);
                        }
                    }
                    pchReqMapper.updatePchReqStCd(userId, pchReqList);
                }

                // 매장 구매요청건에 대한 금액 변동시 출고가능금액 줄여줘야함
                if ("R".equals(savePchOrdRequest.getJobCd())) {
                    if (!"".equals(savePchOrdRequest.getStoreId()) && chgPrAmt > 0) {
                        String loanMgntUnqNo = storeBaseInfoMapper.selectStoreLoanMgntUnqNo(savePchOrdRequest.getStoreId());

                        if (loanMgntUnqNo != null) {
                            loanStateMapper.updateloanMinusPchReqUseAmt(loanMgntUnqNo, savePchOrdRequest.getStoreId(), chgPrAmt);
                        }
                    }
                }


                //출고가능수량을 올려줘야함
                if ("DL01".equals(pchOrdEntity.getDlvDvsn())) { // 본사배송이면
                    //출고창고 값 전달
                    if (pchOrdDtlList.size() > 0) {
                        List<StckMgntBaseInfoEntity> stckList = new ArrayList<StckMgntBaseInfoEntity>();
                        for (int pj = 0 ; pj < pchOrdDtlList.size() ; pj++) {
                            PchOdrDtlPtclEntity poData = pchOrdDtlList.get(pj);
                            StckMgntBaseInfoEntity stckEntity = new StckMgntBaseInfoEntity();
                            stckEntity.setItemId(poData.getItemId());
                            stckEntity.setWaitQty(poData.getPodrQty());
                            stckList.add(stckEntity);
                        }

                        stockMngMapper.updateStckPoUseQty(userId, pchOrdEntity.getBrandId(), pchOrdEntity.getShmtWhsId(), pchOrdEntity.getShmtLocation(), stckList );
                    }
                }
            }

        } catch (GlobalException e) {
            //e.getStackTrace();
            e.printStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
    }


    /*
      구매요청 그리드 리스트 조회
     */
    public GridResponse selectPchOdrGridList(String sBrandId, String fromDt, String toDt, String sPchOdrStCd, String sSpplyId, String sWhsId, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        System.out.println(sBrandId + "==" + fromDt + "==" + toDt + "==" + sPchOdrStCd + "==" + sSpplyId + "==" + sWhsId + "==" + sItemNm);

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = pchOrdMapper.selectPchOdrGridCnt(sBrandId, fromDt, toDt, sPchOdrStCd,sSpplyId,sWhsId,  sItemNm );
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<PchOdrBaseInfoEntity> entities = pchOrdMapper.selectPchOdrGridList(sBrandId, fromDt, toDt, sPchOdrStCd,sSpplyId,sWhsId,  sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> PchOdrSearchListResponse.builder()
                        .viewCd("R")
                        .pchPodrUnqNo(m.getPchPodrUnqNo())
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .pchOdrStCd(m.getPchOdrStCd() == null ? "" : m.getPchOdrStCd())
                        .pchOdrStNm(m.getPchOdrStNm() == null ? "" : m.getPchOdrStNm())
                        .podrDt(m.getPodrDt() == null ? "" : m.getPodrDt())
                        .dlvDvsn(m.getDlvDvsn() == null ? "" : m.getDlvDvsn())
                        .dlvDvsnNm(m.getDlvDvsnNm() == null ? "" : m.getDlvDvsnNm())
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm() + '-' + m.getLocationNm())
                        .locationNm(m.getLocationNm() == null ? "" :  m.getLocationNm())
                        .spplyId(m.getSpplyId() == null ? "" : m.getSpplyId())
                        .spplyNm(m.getSpplyNm() == null ? "" : m.getSpplyNm())
                        .podrTotalAmt(m.getPodrTotalAmt() == null ? 0 : m.getPodrTotalAmt())
                        .podrItemQty(m.getPodrItemQty() == null ? 0 : m.getPodrItemQty())
                        .podrMgrNm(m.getPodrMgrNm() == null ? "" : m.getPodrMgrNm())
                        .shmtWhsNm(m.getShmtWhsNm() == null ? "" : m.getShmtWhsNm())
                        .shmtLocationNm(m.getShmtLocationNm() == null ? "" : m.getShmtLocationNm())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /* 구매발주 정보 */

    public PchOdrInfoResponse selectPchOdrInfo(String  pchReqUnqNo) throws Exception {
        PchOdrBaseInfoEntity entity = pchOrdMapper.selectPchOdrInfo(pchReqUnqNo);
        List<PchOdrDtlPtclEntity> dtlEntity = pchOrdMapper.selectOdrDtlList( pchReqUnqNo);

        return PchOdrInfoResponse.toDTO(entity,dtlEntity);
    }


    /*
      발주 취소처리
     */
    @Transactional
    public void cancelPchOrdDtl(String userId, CancelPchOrdDtlRequest cancelPchOrdDtlRequest) throws Exception {
        try {
            // 품목 삭제 상태처리
            long podrAmt = 0;
            List<PchOdrDtlPtclEntity> gridUpdateData = cancelPchOrdDtlRequest.toCodeEntities(cancelPchOrdDtlRequest.getGridUpdateData());
            List<StckMgntBaseInfoEntity> stckList = new ArrayList<StckMgntBaseInfoEntity>();
            if (gridUpdateData.size() > 0) {
                pchOrdMapper.updateCancelPodrDtl(userId,  gridUpdateData) ;

                for (int i = 0 ; i < gridUpdateData.size() ; i++) {
                    PchOdrDtlPtclEntity delItem = gridUpdateData.get(i);
                    podrAmt = podrAmt + delItem.getPodrAmt();

                    if ("DL01".equals(cancelPchOrdDtlRequest.getDlvDvsn())) { // 본사배송이면
                        StckMgntBaseInfoEntity stckEntity = new StckMgntBaseInfoEntity();
                        stckEntity.setItemId(delItem.getItemId());
                        stckEntity.setWaitQty((-1)*delItem.getPodrQty());
                        stckList.add(stckEntity);
                    }
                }
            }

            //발주취소로 인한 발주 head 상태값 변경처리  pchPodrUnqNo
            String pchOdrStCd = selectOdrDtlStatusList(cancelPchOrdDtlRequest.getPchPodrUnqNo());
            pchOrdMapper.updateChgPodrStCd(userId, pchOdrStCd, cancelPchOrdDtlRequest.getPchPodrUnqNo());

            // 매장 구매요청 발주일경우 사용금액 차감 처리함
            if (!"".equals(cancelPchOrdDtlRequest.getStoreId())) {
                String loanMgntUnqNo = storeBaseInfoMapper.selectStoreLoanMgntUnqNo(cancelPchOrdDtlRequest.getStoreId());

                if (loanMgntUnqNo != null) {
                    loanStateMapper.updateloanMinusPchReqUseAmt(loanMgntUnqNo, cancelPchOrdDtlRequest.getStoreId(), podrAmt);
                }
            }

            //출고가능수량을 복구처리
            if ("DL01".equals(cancelPchOrdDtlRequest.getDlvDvsn())) { // 본사배송이면
                 stockMngMapper.updateStckPoUseQty(userId, cancelPchOrdDtlRequest.getBrandId(), cancelPchOrdDtlRequest.getShmtWhsId(), cancelPchOrdDtlRequest.getShmtLocation(), stckList );
            }

        } catch (GlobalException e) {
            //e.getStackTrace();
            e.printStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
    }

    public String  selectOdrDtlStatusList(String  pchPodrUnqNo) throws Exception {
        //PCH_ODR_ST_CD
        String pchOdrStCd = "";
        List<PchOdrDtlPtclEntity> dtlEntity = pchOrdMapper.selectOdrDtlStatusList( pchPodrUnqNo);

        int pod0 = 0 ;
        int pod1 = 0 ;
        int pod2 = 0 ;
        int pod4 = 0 ;

        for (int i = 0 ; i < dtlEntity.size() ; i++ ) {
            PchOdrDtlPtclEntity entity = dtlEntity.get(i);
            if ( "POD0".equals(entity.getPodrDtlStCd())) {  // 요청
                pod0 = pod0 + entity.getPodrStatusCnt() ;
            } else if ( "POD1".equals(entity.getPodrDtlStCd()) ||  "POD3".equals(entity.getPodrDtlStCd())) {  // 접수 ,출하
                pod1 = pod1 + entity.getPodrStatusCnt() ;
            } else if ( "POD2".equals(entity.getPodrDtlStCd())) {  //발주취소
                pod2 = pod2 + entity.getPodrStatusCnt() ;
            } else if ( "POD4".equals(entity.getPodrDtlStCd())) {  //입고완료
                pod4 = pod4 + entity.getPodrStatusCnt() ;
            }
        }

        if (pod0 > 0 ) {
            pchOdrStCd = "PO20";  //요청
        } else if ( pod0 == 0 && pod1 == 0 &&  pod4 == 0 ) {
            pchOdrStCd = "PO99";  // 취소
        } else {
            pchOdrStCd = "PO30";  // 접수
        }

        return pchOdrStCd;
    }

    /*
     공급사 수주현황 리스트
    */
    public GridResponse selectSpplyPoGridList(String sBrandId, String fromDt, String toDt, String sSpplyId, String sWhsId, String sPchOdrStCd, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = pchOrdMapper.selectSpplyPoGridCnt(sBrandId, fromDt, toDt, sSpplyId, sWhsId, sPchOdrStCd, sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<PchOdrBaseInfoEntity> entities = pchOrdMapper.selectSpplyPoGridList(sBrandId, fromDt, toDt, sSpplyId, sWhsId, sPchOdrStCd, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> SpplyPoOdrSearchListResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .pchPodrUnqNo(m.getPchPodrUnqNo() == null ? "" : m.getPchPodrUnqNo())
                        .pchOdrStCd(m.getPchOdrStCd() == null ? "" : m.getPchOdrStCd())
                        .pchOdrStNm(m.getPchOdrStNm() == null ? "" : m.getPchOdrStNm())
                        .podrDt(m.getPodrDt() == null ? "" : m.getPodrDt())
                        .dlvReqDt(m.getDlvReqDt() == null ? "" : m.getDlvReqDt())
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm() + "-" + m.getLocationNm())
                        .podrTotalAmt(m.getPodrTotalAmt() == null ? 0 : m.getPodrTotalAmt())
                        .podrItemQty(m.getPodrItemQty() == null ? 0 : m.getPodrItemQty())
                        .podrMgrNm(m.getPodrMgrNm() == null ? "" : m.getPodrMgrNm())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /* 구매발주 정보 */

    public SpplyPchOdrInfoResponse selectSpplyPchOdrInfo(String brandId, String  pchReqUnqNo, String spplyId) throws Exception {
        PchOdrBaseInfoEntity entity = pchOrdMapper.selectSpplyPchOdrInfo(brandId, pchReqUnqNo, spplyId);
        List<PchOdrDtlPtclEntity> dtlEntity = pchOrdMapper.selectSpplyOdrDtlList(brandId,  pchReqUnqNo, spplyId);

        return SpplyPchOdrInfoResponse.toDTO(entity,dtlEntity);
    }


    /*
          간소화 구매 발주 정보 저장
         */
    @Transactional
    public void savePchSteamlineOrdInfo(String userId, SavePchSteamlineOrdRequest savePchSteamlineOrdRequest) throws Exception {
        try {
            // 로그인 정보 가져오기
            UserMngEntity uEntity = userMngMapper.selectGroupUserInfo(userId);
            uEntity.setTelNo(uEntity.getTelNo().replaceAll("-",""));

            // 납품창고의 연락처 정보 가져오기

            // 구매요청건별 발주 진행 처리함
            List<PchReqBaseInfoEntity> gridInsert = savePchSteamlineOrdRequest.toCodeEntities(savePchSteamlineOrdRequest.getGridInsertData());

            // 전달받은 구매요청건에 대해 재정리
            List<PchReqBaseInfoEntity> gridInsertData = pchReqMapper.selectPchPoStreamlineReqList(savePchSteamlineOrdRequest.getBrandId(), gridInsert);

            // 헤더데이타 체크
            if (gridInsertData.size() < 1) {
                throw new GlobalException(SystemMessageCode.ERR_NOTITEM);
            }

            // 기본창고 여부 확인 (본사배송 건 존재시 기본창고 여부 확인 하여 없을 경우 오류 처리 있으면 그냥 진행
            int brandDlvCnt = pchOrdMapper.selectPchPoStreamlineBrandDlvChk(savePchSteamlineOrdRequest.getBrandId(), gridInsertData);
            WhsMgntBaseInfoEntity whsEntity = null ;

            if (brandDlvCnt > 0 ) {
                // 기본창고 및 로케이션 있는지 여부 확인
                int baseWhsCnt = whsMngMapper.selectBaseWhsLocationCnt(savePchSteamlineOrdRequest.getBrandId()) ;

                if (baseWhsCnt > 0 ) {
                    whsEntity = whsMngMapper.selectBaseWhsLocationInfo(savePchSteamlineOrdRequest.getBrandId()) ;
                } else {
                    //오류 처리 (창고관리에서 기본창고를 지정해주세요!)
                    throw new GlobalException(SystemMessageCode.ERR_NON_BASE_WHS);
                }
            }

            //처리 내역 저장 구분
            PchOdrBaseInfoEntity tempTargetBase =  null ; //new PchOdrBaseInfoEntity();
            List<PchOdrBaseInfoEntity> tempTargetBaseList = new ArrayList<PchOdrBaseInfoEntity>();
            PchOdrDtlPtclEntity tempTargetPtcl = null ; //new PchOdrDtlPtclEntity();
            List<PchOdrDtlPtclEntity> tempTargetPtclList = new ArrayList<PchOdrDtlPtclEntity>();

            ShmtOdrBaseEntity shmtBaseEntity = null ; //new ShmtOdrBaseEntity();
            List<ShmtOdrBaseEntity> shmtBaseList = new ArrayList<ShmtOdrBaseEntity>();
            ShmtOdrDtlEntity shmtOdrDtlEntity = null ; //new shmtOdrDtlEntity();
            List<ShmtOdrDtlEntity> shmtDtlList = new ArrayList<ShmtOdrDtlEntity>();

            DeliInfoMgntEntity deliBaseEntity = null; //new DeliInfoMgntEntity();
            List<DeliInfoMgntEntity> deliBaseList = new ArrayList<DeliInfoMgntEntity>();
            DeliDtlMgntEntity deliDtlEntity = null ; //new DeliDtlMgntEntity();
            List<DeliDtlMgntEntity> deliDtlList = new ArrayList<DeliDtlMgntEntity>();

            StckMgntBaseInfoEntity stckEntity = null;
            List<StckMgntBaseInfoEntity> stockList = new ArrayList<StckMgntBaseInfoEntity>();

            for (int i = 0 ; i < gridInsertData.size() ; i++) {
                PchReqBaseInfoEntity pchReqEntity = gridInsertData.get(i);
                // 구매요청번호로 구매요청 내역 가져오기
                List<PchOdrBaseInfoEntity> pchOrdList = pchOrdMapper.selectPchPoStreamlineList(savePchSteamlineOrdRequest.getBrandId(),pchReqEntity.getPchReqUnqNo());

                // 구매발주번호, 출하번호, 납품서번호
                String preDlvDvsn = "";
                String preSpplyId = "";
                String pchPodrUnqNo = "";
                String shmtReqUnqNo = ""; // 출하지시번호
                String dlvSttmtUnqNo = ""; // 납품서번호
                int dtlRow = 0 ;
                long poTotAmt = 0 ;
                int poItemCnt = 0 ;

                for (int j = 0 ; j < pchOrdList.size() ; j++) {
                    // 구매요청건에 대한 개별 처리 방안
                    PchOdrBaseInfoEntity  podrEntity = pchOrdList.get(j);

                    if (preDlvDvsn.equals(podrEntity.getDlvDvsn())) {
                        if ("DL01".equals(podrEntity.getDlvDvsn())) {
                            //본사배송 기존거와 동일
                            tempTargetPtcl = new PchOdrDtlPtclEntity();
                            tempTargetPtcl.setPchPodrUnqNo(pchPodrUnqNo);
                            dtlRow = dtlRow + 10;
                            tempTargetPtcl.setPchPodrDtlNo(Integer.toString(dtlRow));
                            tempTargetPtcl.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                            tempTargetPtcl.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                            tempTargetPtcl.setPodrDtlStCd("POD3");
                            tempTargetPtcl.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                            tempTargetPtcl.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                            tempTargetPtcl.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            tempTargetPtcl.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            tempTargetPtcl.setCurr(podrEntity.getPchOdrDtlInfo().getCurr());
                            tempTargetPtcl.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                            tempTargetPtcl.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                            tempTargetPtcl.setDlvSchDt(podrEntity.getPchOdrDtlInfo().getDlvSchDt());
                            tempTargetPtcl.setStoreUnitUnqNo(podrEntity.getPchOdrDtlInfo().getStoreUnitUnqNo());
                            tempTargetPtcl.setBrandUnitUnqNo(podrEntity.getPchOdrDtlInfo().getBrandUnitUnqNo());
                            tempTargetPtclList.add(tempTargetPtcl);

                            poTotAmt = poTotAmt + podrEntity.getPchOdrDtlInfo().getPodrAmt();
                            poItemCnt = poItemCnt + 1;

                            shmtOdrDtlEntity = new ShmtOdrDtlEntity();
                            shmtOdrDtlEntity.setShmtReqUnqNo(shmtReqUnqNo);
                            //System.out.println("DL01.equals(podrEntity.getDlvDvsn())===================>" + shmtReqUnqNo + "===" + Integer.toString(dtlRow));
                            shmtOdrDtlEntity.setShmtReqDtlNo(Integer.toString(dtlRow));
                            shmtOdrDtlEntity.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                            shmtOdrDtlEntity.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                            shmtOdrDtlEntity.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                            shmtOdrDtlEntity.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                            shmtOdrDtlEntity.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            shmtOdrDtlEntity.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            shmtOdrDtlEntity.setShmtQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            shmtOdrDtlEntity.setShmtAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            shmtOdrDtlEntity.setExpryDate("");
                            shmtOdrDtlEntity.setPchPodrUnqNo(pchPodrUnqNo);
                            shmtOdrDtlEntity.setPchPodrDtlNo(Integer.toString(dtlRow));
                            shmtOdrDtlEntity.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                            shmtOdrDtlEntity.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                            shmtDtlList.add(shmtOdrDtlEntity);

                            stckEntity = new StckMgntBaseInfoEntity();
                            stckEntity.setBrandId(savePchSteamlineOrdRequest.getBrandId());
                            stckEntity.setWhsId(whsEntity.getWhsId());
                            stckEntity.setLocation(whsEntity.getLocationInfo().getLocation());
                            stckEntity.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                            stckEntity.setStckQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            stckEntity.setWaitQty(0);
                            stckEntity.setRelationNo(shmtReqUnqNo);
                            stockList.add(stckEntity);

                            deliDtlEntity = new DeliDtlMgntEntity();
                            deliDtlEntity.setDlvSttmtUnqNo(dlvSttmtUnqNo);
                            deliDtlEntity.setDlvSttmtDtlNo(Integer.toString(dtlRow));
                            deliDtlEntity.setPchPodrUnqNo(pchPodrUnqNo);
                            deliDtlEntity.setPchPodrDtlNo(Integer.toString(dtlRow));
                            deliDtlEntity.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                            deliDtlEntity.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                            deliDtlEntity.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                            deliDtlEntity.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                            deliDtlEntity.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                            deliDtlEntity.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                            deliDtlEntity.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            deliDtlEntity.setDlvQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            deliDtlEntity.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            deliDtlEntity.setDlvAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            deliDtlEntity.setShmtReqUnqNo(shmtReqUnqNo);
                            deliDtlEntity.setShmtReqDtlNo(Integer.toString(dtlRow));
                            deliDtlEntity.setExpryDate(null);
                            deliDtlList.add(deliDtlEntity);

                        } else {
                            if (preSpplyId.equals(podrEntity.getSpplyId())) {
                                //기존거와 동일
                                tempTargetPtcl = new PchOdrDtlPtclEntity();
                                tempTargetPtcl.setPchPodrUnqNo(pchPodrUnqNo);
                                dtlRow = dtlRow + 10;
                                tempTargetPtcl.setPchPodrDtlNo(Integer.toString(dtlRow));
                                tempTargetPtcl.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                                tempTargetPtcl.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                                tempTargetPtcl.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                                tempTargetPtcl.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                                tempTargetPtcl.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                                tempTargetPtcl.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                                tempTargetPtcl.setCurr(podrEntity.getPchOdrDtlInfo().getCurr());
                                tempTargetPtcl.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                                tempTargetPtcl.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                                tempTargetPtcl.setDlvSchDt(podrEntity.getPchOdrDtlInfo().getDlvSchDt());
                                tempTargetPtcl.setStoreUnitUnqNo(podrEntity.getPchOdrDtlInfo().getStoreUnitUnqNo());
                                tempTargetPtcl.setBrandUnitUnqNo(podrEntity.getPchOdrDtlInfo().getBrandUnitUnqNo());
                                tempTargetPtcl.setPodrDtlStCd("POD0");
                                tempTargetPtclList.add(tempTargetPtcl);

                                poTotAmt = poTotAmt + podrEntity.getPchOdrDtlInfo().getPodrAmt();
                                poItemCnt = poItemCnt + 1;
                            } else {
                                //기존거와 상이
                                if (!"".toString().equals(preDlvDvsn)) {
                                    tempTargetBase.setPodrTotalAmt(poTotAmt);
                                    tempTargetBase.setPodrItemQty(poItemCnt);
                                    tempTargetBaseList.add(tempTargetBase);

                                    if ("DL01".equals(preDlvDvsn)) {
                                        shmtBaseEntity.setShmtTotalAmt(poTotAmt);
                                        shmtBaseEntity.setItemQty(poItemCnt);
                                        deliBaseEntity.setDlvAmt(poTotAmt);
                                        shmtBaseList.add(shmtBaseEntity);
                                        deliBaseList.add(deliBaseEntity);
                                    }
                                }

                                poTotAmt = 0 ;
                                poItemCnt = 0 ;
                                tempTargetBase = new PchOdrBaseInfoEntity();
                                tempTargetPtcl = new PchOdrDtlPtclEntity();
                                preSpplyId = podrEntity.getSpplyId();

                                pchPodrUnqNo = fnGetDealNo(14,"");
                                tempTargetBase.setPchPodrUnqNo(pchPodrUnqNo);
                                preDlvDvsn = podrEntity.getDlvDvsn();
                                preSpplyId = podrEntity.getSpplyId();
                                tempTargetBase.setBrandId(savePchSteamlineOrdRequest.getBrandId());
                                tempTargetBase.setStoreId(podrEntity.getStoreId());
                                tempTargetBase.setSpplyId(podrEntity.getSpplyId());

                                tempTargetBase.setDlvDvsn(podrEntity.getDlvDvsn());
                                tempTargetBase.setDlvReqDt(podrEntity.getDlvReqDt());
                                tempTargetBase.setPodrMgrNm(uEntity.getUserNm());
                                tempTargetBase.setPodrDvsn(podrEntity.getPodrDvsn());
                                tempTargetBase.setPodrMgrId(uEntity.getUserId());
                                tempTargetBase.setPodrMgrTelNo(uEntity.getTelNo());
                                tempTargetBase.setWhsDvsn(podrEntity.getWhsDvsn());
                                tempTargetBase.setWhsId(podrEntity.getWhsId());
                                tempTargetBase.setLocation(podrEntity.getLocation());
                                tempTargetBase.setPchOdrStCd("PO20");

                                tempTargetPtcl.setPchPodrUnqNo(pchPodrUnqNo);
                                dtlRow = 10;
                                // String dtlNo = Integer.toString(dtlRow);
                                tempTargetPtcl.setPchPodrDtlNo(Integer.toString(dtlRow));
                                tempTargetPtcl.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                                tempTargetPtcl.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                                tempTargetPtcl.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                                tempTargetPtcl.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                                tempTargetPtcl.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                                tempTargetPtcl.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                                tempTargetPtcl.setCurr(podrEntity.getPchOdrDtlInfo().getCurr());
                                tempTargetPtcl.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                                tempTargetPtcl.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                                tempTargetPtcl.setDlvSchDt(podrEntity.getPchOdrDtlInfo().getDlvSchDt());
                                tempTargetPtcl.setStoreUnitUnqNo(podrEntity.getPchOdrDtlInfo().getStoreUnitUnqNo());
                                tempTargetPtcl.setBrandUnitUnqNo(podrEntity.getPchOdrDtlInfo().getBrandUnitUnqNo());
                                tempTargetPtcl.setPodrDtlStCd("POD0");
                                tempTargetPtclList.add(tempTargetPtcl);

                                poTotAmt = poTotAmt + podrEntity.getPchOdrDtlInfo().getPodrAmt();
                                poItemCnt = poItemCnt + 1;
                            }
                        }
                    } else {
                        // 서로 상이할경우
                        if (!"".toString().equals(preDlvDvsn)) {
                            tempTargetBase.setPodrTotalAmt(poTotAmt);
                            tempTargetBase.setPodrItemQty(poItemCnt);
                            tempTargetBaseList.add(tempTargetBase);

                            if ("DL01".equals(preDlvDvsn)) {
                                shmtBaseEntity.setShmtTotalAmt(poTotAmt);
                                shmtBaseEntity.setItemQty(poItemCnt);
                                deliBaseEntity.setDlvAmt(poTotAmt);
                                shmtBaseList.add(shmtBaseEntity);
                                deliBaseList.add(deliBaseEntity);
                            }
                        }

                        poTotAmt = 0 ;
                        poItemCnt = 0 ;
                        tempTargetBase = new PchOdrBaseInfoEntity();
                        tempTargetPtcl = new PchOdrDtlPtclEntity();


                        // 이미등록건 업데이트 처리
                        pchPodrUnqNo = fnGetDealNo(14,"");
                        tempTargetBase.setPchPodrUnqNo(pchPodrUnqNo);
                        preDlvDvsn = podrEntity.getDlvDvsn();
                        preSpplyId = podrEntity.getSpplyId();
                        tempTargetBase.setBrandId(savePchSteamlineOrdRequest.getBrandId());
                        tempTargetBase.setStoreId(podrEntity.getStoreId());

                        if ("DL01".equals(podrEntity.getDlvDvsn())) {
                            tempTargetBase.setSpplyId(null);
                        } else {
                            tempTargetBase.setSpplyId(podrEntity.getSpplyId());
                        }

                        tempTargetBase.setDlvDvsn(podrEntity.getDlvDvsn());
                        tempTargetBase.setDlvReqDt(podrEntity.getDlvReqDt());
                        tempTargetBase.setPodrMgrNm(uEntity.getUserNm());
                        tempTargetBase.setPodrDvsn(podrEntity.getPodrDvsn());
                        tempTargetBase.setPodrMgrId(uEntity.getUserId());
                        tempTargetBase.setPodrMgrTelNo(uEntity.getTelNo());
                        tempTargetBase.setWhsDvsn(podrEntity.getWhsDvsn());
                        tempTargetBase.setWhsId(podrEntity.getWhsId());
                        tempTargetBase.setLocation(podrEntity.getLocation());

                        tempTargetPtcl.setPchPodrUnqNo(pchPodrUnqNo);
                        dtlRow = 10;
                       // String dtlNo = Integer.toString(dtlRow);
                        tempTargetPtcl.setPchPodrDtlNo(Integer.toString(dtlRow));
                        tempTargetPtcl.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                        tempTargetPtcl.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                        tempTargetPtcl.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                        tempTargetPtcl.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                        tempTargetPtcl.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                        tempTargetPtcl.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                        tempTargetPtcl.setCurr(podrEntity.getPchOdrDtlInfo().getCurr());
                        tempTargetPtcl.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                        tempTargetPtcl.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                        tempTargetPtcl.setDlvSchDt(podrEntity.getPchOdrDtlInfo().getDlvSchDt());
                        tempTargetPtcl.setStoreUnitUnqNo(podrEntity.getPchOdrDtlInfo().getStoreUnitUnqNo());
                        tempTargetPtcl.setBrandUnitUnqNo(podrEntity.getPchOdrDtlInfo().getBrandUnitUnqNo());
                        tempTargetPtclList.add(tempTargetPtcl);

                        poTotAmt = poTotAmt + podrEntity.getPchOdrDtlInfo().getPodrAmt();
                        poItemCnt = poItemCnt + 1;

                        //본사배송일경우 출하지시 , 납품서 생성 처리함
                        if ("DL01".equals(podrEntity.getDlvDvsn())) {
                            tempTargetBase.setShmtWhsId(whsEntity.getWhsId());
                            tempTargetBase.setShmtLocation(whsEntity.getLocationInfo().getLocation());
                            tempTargetBase.setPchOdrStCd("PO30");
                            tempTargetPtcl.setPodrDtlStCd("POD3");

                            shmtBaseEntity = new ShmtOdrBaseEntity();
                            shmtOdrDtlEntity = new ShmtOdrDtlEntity();
                            deliBaseEntity = new DeliInfoMgntEntity();
                            deliDtlEntity = new DeliDtlMgntEntity();

                            // 출하지시 처리
                            shmtReqUnqNo = fnGetDealNo(16, "");  // 출하지시번호
                            shmtBaseEntity.setShmtReqUnqNo(shmtReqUnqNo);
                            shmtBaseEntity.setShmtReqStCd("SHM1");
                            shmtBaseEntity.setShmtWhsId(whsEntity.getWhsId());
                            shmtBaseEntity.setShmtLocation(whsEntity.getLocationInfo().getLocation());
                            shmtBaseEntity.setWhsDvsn("W01");
                            shmtBaseEntity.setBrandId(savePchSteamlineOrdRequest.getBrandId());
                            shmtBaseEntity.setStoreId(podrEntity.getStoreId());
                            shmtBaseEntity.setPodrDvsn("OD01");
                            shmtBaseEntity.setPodrMgrNm(uEntity.getUserNm());
                            shmtBaseEntity.setDlvReqDt(podrEntity.getDlvReqDt());
                            shmtBaseEntity.setWhsId(podrEntity.getWhsId());
                            shmtBaseEntity.setLocation(podrEntity.getLocation());

                            shmtOdrDtlEntity.setShmtReqUnqNo(shmtReqUnqNo);
                            System.out.println("NEW===================>" + shmtReqUnqNo + "===" + Integer.toString(dtlRow));
                            shmtOdrDtlEntity.setShmtReqDtlNo(Integer.toString(dtlRow));
                            shmtOdrDtlEntity.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                            shmtOdrDtlEntity.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                            shmtOdrDtlEntity.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                            shmtOdrDtlEntity.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                            shmtOdrDtlEntity.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            shmtOdrDtlEntity.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            shmtOdrDtlEntity.setShmtQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            shmtOdrDtlEntity.setShmtAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            shmtOdrDtlEntity.setExpryDate("");
                            shmtOdrDtlEntity.setPchPodrUnqNo(pchPodrUnqNo);
                            shmtOdrDtlEntity.setPchPodrDtlNo(Integer.toString(dtlRow));
                            shmtOdrDtlEntity.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                            shmtOdrDtlEntity.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                            shmtDtlList.add(shmtOdrDtlEntity);

                            stckEntity = new StckMgntBaseInfoEntity();
                            stckEntity.setBrandId(savePchSteamlineOrdRequest.getBrandId());
                            stckEntity.setWhsId(whsEntity.getWhsId());
                            stckEntity.setLocation(whsEntity.getLocationInfo().getLocation());
                            stckEntity.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                            stckEntity.setStckQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            stckEntity.setWaitQty(0);
                            stckEntity.setRelationNo(shmtReqUnqNo);
                            stockList.add(stckEntity);

                            // 납품서 처리
                            dlvSttmtUnqNo = fnGetDealNo(15, ""); // 납품서번호
                            deliBaseEntity.setDlvSttmtUnqNo(dlvSttmtUnqNo);
                            deliBaseEntity.setSpplyId(null);
                            deliBaseEntity.setSpplyNm(null);
                            deliBaseEntity.setStoreId(podrEntity.getStoreId());
                            deliBaseEntity.setDlvMgrNm(uEntity.getUserNm());
                            deliBaseEntity.setDlvMgrTelNo(uEntity.getTelNo());
                            deliBaseEntity.setDlvMgrEmal(null);
                            deliBaseEntity.setDlvMgrMblNo(uEntity.getTelNo());
                            deliBaseEntity.setVndrTelNo(whsEntity.getMgrTelNo());
                            deliBaseEntity.setDlvStCd("DLV0");
                            deliBaseEntity.setPodrRegDt(podrEntity.getPodrDt());
                            deliBaseEntity.setDlvReqDt(podrEntity.getPodrDt());
                            deliBaseEntity.setDlvSchDt(podrEntity.getPodrDt());
                            deliBaseEntity.setDlvCmpltDt(null);
                            deliBaseEntity.setWhsId(podrEntity.getWhsId());
                            deliBaseEntity.setLocation(podrEntity.getLocation());
                            deliBaseEntity.setWhsDvsn("W02");
                            deliBaseEntity.setBrandId(savePchSteamlineOrdRequest.getBrandId());

                            deliDtlEntity.setDlvSttmtUnqNo(dlvSttmtUnqNo);
                            deliDtlEntity.setDlvSttmtDtlNo(Integer.toString(dtlRow));
                            deliDtlEntity.setPchPodrUnqNo(pchPodrUnqNo);
                            deliDtlEntity.setPchPodrDtlNo(Integer.toString(dtlRow));
                            deliDtlEntity.setPchReqUnqNo(podrEntity.getPchOdrDtlInfo().getPchReqUnqNo());
                            deliDtlEntity.setPchReqDtlNo(podrEntity.getPchOdrDtlInfo().getPchReqDtlNo());
                            deliDtlEntity.setItemId(podrEntity.getPchOdrDtlInfo().getItemId());
                            deliDtlEntity.setItemNm(podrEntity.getPchOdrDtlInfo().getItemNm());
                            deliDtlEntity.setUnit(podrEntity.getPchOdrDtlInfo().getUnit());
                            deliDtlEntity.setUnitAmt(podrEntity.getPchOdrDtlInfo().getUnitAmt());
                            deliDtlEntity.setPodrQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            deliDtlEntity.setDlvQty(podrEntity.getPchOdrDtlInfo().getPodrQty());
                            deliDtlEntity.setPodrAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            deliDtlEntity.setDlvAmt(podrEntity.getPchOdrDtlInfo().getPodrAmt());
                            deliDtlEntity.setShmtReqUnqNo(shmtReqUnqNo);
                            deliDtlEntity.setShmtReqDtlNo(Integer.toString(dtlRow));
                            deliDtlEntity.setExpryDate(null);
                            deliDtlList.add(deliDtlEntity);

                        } else {
                            tempTargetBase.setPchOdrStCd("PO20");
                            tempTargetPtcl.setPodrDtlStCd("POD0");
                        }
                    }

                    //마지막 부분 추가 하기
                    if (j == pchOrdList.size()-1) {
                        tempTargetBase.setPodrTotalAmt(poTotAmt);
                        tempTargetBase.setPodrItemQty(poItemCnt);
                        tempTargetBaseList.add(tempTargetBase);
                        if ("DL01".equals(preDlvDvsn)) {
                            shmtBaseEntity.setShmtTotalAmt(poTotAmt);
                            shmtBaseEntity.setItemQty(poItemCnt);
                            shmtBaseList.add(shmtBaseEntity);
                            deliBaseList.add(deliBaseEntity);
                        }
                    }

                }  // 구매요청건 개별 종료

            } // for문 종료

            // 발주 생성
            pchOrdMapper.insertStreamListPchOdrBaseInfo(userId, tempTargetBaseList);
            pchOrdMapper.insertStreamListPchOdrDtlPtcl(userId, tempTargetPtclList);
            //구매요청 상태 업데이트
            pchReqMapper.updateStreamListPchReqDtlStCd(userId,gridInsertData );
            pchReqMapper.updateStreamListPchReqStCd(userId,gridInsertData );

            // 출하지시생성
            if (shmtBaseList.size() > 0 ) {
                // 출하지시, 납품서 등록 및 재고 처리
                shmtOdrMapper.insertStreamListShmtOdrBase(userId, shmtBaseList);
                shmtOdrMapper.insertShmtOdrDtl(userId, shmtDtlList);
                shmtOdrMapper.insertStreamListDeliInfoMngt(userId, deliBaseList);
                shmtOdrMapper.insertDeliDtlPtcl(userId, deliDtlList);

                stockMngMapper.updateMinStockQty(userId, stockList);
                List<StckMgntDtlHistEntity> stockHistList = new ArrayList<StckMgntDtlHistEntity>();
                for (int m = 0; m < stockList.size(); m++) {
                    StckMgntBaseInfoEntity baseEntity = stockList.get(m);
                    StckMgntDtlHistEntity sourceHist = new StckMgntDtlHistEntity();
                    sourceHist.setStckMgntDtlNo(fnGetDealNo(32, "")); // 재고관리상세번호
                    sourceHist.setBrandId(baseEntity.getBrandId());
                    sourceHist.setWhsId(baseEntity.getWhsId());
                    sourceHist.setLocation(baseEntity.getLocation());
                    sourceHist.setItemId(baseEntity.getItemId());
                    sourceHist.setItemNm(baseEntity.getItemNm());
                    sourceHist.setInoutCd(InOutCd.OUT.getCd()); // 출고
                    sourceHist.setInoutQty(baseEntity.getStckQty());
                    sourceHist.setChgDvsn(ChgDvsn.PURCHASE_OUT.getCd()); // 구매출고
                    sourceHist.setRelationNo(baseEntity.getRelationNo());
                    stockHistList.add(sourceHist);
                }
                stockMngMapper.insertMultiStckMgntDtlHist(userId, stockHistList);
            }



       } catch (GlobalException e) {
            //e.getStackTrace();
            e.printStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
    }


}
