package com.oqpo.api.web.controller.stockmng;

import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.service.stockmng.StockMngService;
import com.oqpo.api.web.dto.request.stockmng.*;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.SaveResponse;
import com.oqpo.api.web.dto.response.stockmng.BarcodeWhsItemInfoResponse;
import com.oqpo.api.web.dto.response.stockmng.StockMngInfoResponse;
import com.oqpo.api.web.dto.response.stockmng.StockMngPassqtyListResponse;
import com.oqpo.api.web.dto.response.stockmng.WhsItemInfoResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;

@Slf4j
@RestController
@RequestMapping("/api/stock/mng")
@Api(tags = {"재고관리 (StockMngController) -- 완료"})
public class StockMngController {

    @Autowired
    private StockMngService stockMngService;

    /**
     * 설명 : 재고관리 그리드 리스트
     *
     * @param stockMngGridRequest
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "재고관리 그리드 리스트")
    @PostMapping("/detail-grid-list")
    public ResponseEntity<GridResponse> detailGridList(@RequestBody @Valid StockMngGridRequest stockMngGridRequest) throws Exception {
        return ResponseEntity.ok(stockMngService.selectStockMngGridList(stockMngGridRequest.getSBrandId(), stockMngGridRequest.getSStoreId(),
                stockMngGridRequest.getSWhsId(), stockMngGridRequest.getSLocation(),
                stockMngGridRequest.getSItemStCd(), stockMngGridRequest.getSItemType(), stockMngGridRequest.getSItemGubun(),
                stockMngGridRequest.getSItemClass1(), stockMngGridRequest.getSItemClass2(), stockMngGridRequest.getSItemClass3(), stockMngGridRequest.getSItemClass4(),
                stockMngGridRequest.getSItemId(), stockMngGridRequest.getSItemNm(), stockMngGridRequest.toDTO(stockMngGridRequest)));
    }

    /**
     * 설명 : 재고실사 그리드 리스트 - 엑셀업로드 체크
     *
     * @param stockMngCddGridRequest
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "재고실사 그리드 리스트 - 엑셀업로드 체크")
    @PostMapping("/cdd-grid-list")
    public ResponseEntity<GridResponse> cddGridList(@RequestBody @Valid StockMngCddGridRequest stockMngCddGridRequest) throws Exception {
        return ResponseEntity.ok(stockMngService.selectStockCddGridList(stockMngCddGridRequest, stockMngCddGridRequest.toDTO(stockMngCddGridRequest)));
    }

    /**
     * 설명 : 재고 정보
     *
     * @param brandId
     * @param whsId
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "brandId", value = "브랜드아이디", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "whsId", value = "창고아이디", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "location", value = "Location", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "itemId", value = "품목아이디", required = true, dataType = "String", paramType = "query")
    })
    @ApiOperation(value = "재고 정보")
    @GetMapping("/info-stock")
    public ResponseEntity<StockMngInfoResponse> infoStock(@RequestParam(value = "brandId") String brandId, @RequestParam(value = "whsId") String whsId,
                                                          @RequestParam(value = "location") String location, @RequestParam(value = "itemId") String itemId) throws Exception {
        return ResponseEntity.ok(stockMngService.selectStockMngInfo(brandId, whsId, location, itemId));
    }

    /**
     * 설명 : 재고이력 그리드 리스트
     *
     * @param stockHistGridRequest
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "재고이력 그리드 리스트")
    @PostMapping("/hist-grid-list")
    public ResponseEntity<GridResponse> histGridList(@RequestBody @Valid StockHistGridRequest stockHistGridRequest) throws Exception {
        return ResponseEntity.ok(stockMngService.selectStockHistGridList(stockHistGridRequest.getSBrandId(), stockHistGridRequest.getSWhsId(),
                stockHistGridRequest.getSLocation(), stockHistGridRequest.getSItemId(), stockHistGridRequest.getSInoutCd(),
                stockHistGridRequest.getFromDt(), stockHistGridRequest.getToDt(),
                stockHistGridRequest.toDTO(stockHistGridRequest)));
    }

    /**
     * 설명 : 재고 처리
     *
     * @param userId
     * @param procStockRequest
     * @return SaveResponse
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "재고 처리")
    @PostMapping("/proc-stock")
    public ResponseEntity<SaveResponse> procStock(@ApiIgnore String userId, @RequestBody @Valid ProcStockRequest procStockRequest) throws Exception {
        stockMngService.procStock(userId, procStockRequest);
        return ResponseEntity.ok(SaveResponse.toDTO(SystemMessageCode.SAVE_OK));
    }


    /**
     * 설명 : 창고별 출고가능 수량 정보 출력
     *
     * @param userId
     * @param StockMngPassQtyGridRequest
     * @return SaveResponse
     * @throws Exception
     */

    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "창고별 출고가능 수량 정보 출력")
    @PostMapping("/whs-passqty-list")
    public ResponseEntity<StockMngPassqtyListResponse> selectWhsLocOutPassQty(@ApiIgnore String userId, @RequestBody @Valid StockMngPassQtyGridRequest stockMngPassQtyGridRequest) throws Exception {
        return ResponseEntity.ok(stockMngService.selectWhsLocOutPassQty(userId, stockMngPassQtyGridRequest));

    }


    /**
     * 설명 : 창고별 품목 조회 팝업
     *
     * @param stockMngGridRequest
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "창고별 품목 조회 팝업")
    @PostMapping("/pop-whs-item-grid-list")
    public ResponseEntity<GridResponse> popWhsItemGridList(@RequestBody @Valid PopWhsItemStockGridRequest popWhsItemStockGridRequest) throws Exception {
        return ResponseEntity.ok(stockMngService.selectPopItemStockMngGridList(popWhsItemStockGridRequest.getSBrandId(),
                popWhsItemStockGridRequest.getSWhsId(), popWhsItemStockGridRequest.getSLocation(),
                popWhsItemStockGridRequest.getSItemNm(), popWhsItemStockGridRequest.toDTO(popWhsItemStockGridRequest)));
    }

    /**
     * 설명 : 바코드를 통한 창고 품목 정보조회
     *
     * @param brandId
     * @param whsId
     * @return
     * @throws Exception
     */
    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "sBrandId", value = "브랜드아이디", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "sWhsId", value = "창고아이디", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "sLocation", value = "Location", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "sBarcode", value = "바코드", required = true, dataType = "String", paramType = "query")
    })
    @ApiOperation(value = "바코드를 통한 창고 품목 정보조회")
    @GetMapping("/info-barcode-stock")
    public ResponseEntity<BarcodeWhsItemInfoResponse> infoBarcodeStock(@RequestParam(value = "sBrandId") String sBrandId, @RequestParam(value = "sWhsId") String sWhsId,
                                                                       @RequestParam(value = "sLocation") String sLocation, @RequestParam(value = "sBarcode") String sBarcode) throws Exception {
        return ResponseEntity.ok(stockMngService.selectBarcodeWhsItemMngInfo(sBrandId, sWhsId, sLocation, sBarcode));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "창고 재고 품목 정보조회")
    @PostMapping("/info-whs-item")
    public ResponseEntity<GridResponse> selectWhsLocationItemList(@RequestBody @Valid WhsLocationItemListRequest whsLocationItemListRequest) throws Exception {
        return ResponseEntity.ok(stockMngService.selectWhsLocationItemList(whsLocationItemListRequest.getBrandId(), whsLocationItemListRequest.getWhsId(), whsLocationItemListRequest.getLocation(), whsLocationItemListRequest.toDTO(whsLocationItemListRequest)));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "X-AUTH-TOKEN", value = "CONN-KEY", required = true, dataType = "String", paramType = "header")
    })
    @ApiOperation(value = "창고 재고 품목 엑셀 다운로드")
    @PostMapping("/info-whs-item-excel")
    public ResponseEntity<GridResponse> info_whs_item_excel(@RequestBody @Valid WhsLocationItemListRequest whsLocationItemListRequest) throws Exception {
        return ResponseEntity.ok(stockMngService.selectWhsLocationItemList4Excel(whsLocationItemListRequest.getBrandId(), whsLocationItemListRequest.getWhsId(), whsLocationItemListRequest.getLocation(), whsLocationItemListRequest.toDTO(whsLocationItemListRequest)));
    }

}
