package com.oqpo.api.web.dto.response.session;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.oqpo.api.entity.*;
import com.oqpo.api.entity.oper.AfflEntity;
import com.oqpo.api.entity.oper.MemberEntity;
import com.oqpo.api.entity.oper.MenuEntity;
import com.oqpo.api.entity.oper.PopEntity;
import com.oqpo.api.entity.CodeEntity;
import com.oqpo.api.entity.stinfo.BrandEntity;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.stream.Collectors;

@Getter
@Setter
@Builder
@AllArgsConstructor(access = AccessLevel.PROTECTED)
@JsonInclude(JsonInclude.Include.ALWAYS)
public class SessionResponse {
    @ApiModelProperty(value = "사용자정보")
    private UserInfo userInfo;

    @ApiModelProperty(value = "코드 리스트")
    private List<CodeInfo> codeList;

    @ApiModelProperty(value = "메뉴 리스트")
    private List<AuthMenuInfo> authMenuList;

    @ApiModelProperty(value = "액션 리스트")
    private List<ActInfo> actList;

    public static SessionResponse toDTO(MemberEntity entity, List<CodeEntity> codeList, List<MenuEntity> authMenuList, List<ActionEntity> actList) {
        if (entity == null) return null;
        return SessionResponse.builder()
                .userInfo(UserInfo.builder()
                        .userId(entity.getUserId())
                        .userNm(entity.getUserNm())
                        .authTpCd(entity.getAuthTpCd())
                        .authTpNm(entity.getAuthTpNm())
                        .afflShopId(entity.getAfflShopId() == null ? "" :entity.getAfflShopId() )
                        .afflShopNm(entity.getAfflShopNm() == null ? "" : entity.getAfflShopNm())
                        .brandId(entity.getBrandId() == null ? "" : entity.getBrandId() )
                        .brandNm(entity.getBrandNm() == null ? "" : entity.getBrandNm())
                        .storeId(entity.getStoreId() == null ? "" : entity.getStoreId())
                        .storeNm(entity.getStoreNm() == null ? "" : entity.getStoreNm())
                        .spplyId(entity.getSpplyId() == null ? "" : entity.getSpplyId())
                        .spplyNm(entity.getSpplyNm() == null ? "" : entity.getSpplyNm())
                        .setSystemAdminYn(StringUtils.isBlank(entity.getSystemAdminYn()) ? "N" : entity.getSystemAdminYn())
                        .oqBrandId(entity.getOqBrandId() == null ? "" : entity.getOqBrandId())
                        .oqStoreId(entity.getOqStoreId() == null ? "" : entity.getOqStoreId())
                        .apprYn(entity.getApprYn() == null ? "N" : entity.getApprYn())
                        .mallId(entity.getMallId() == null ? "" : entity.getMallId())
                        .shmtColProcType(entity.getShmtColProcType() == null ? "0" : entity.getShmtColProcType())
                        .build())
                .codeList(codeList.stream()
                        .map(m -> CodeInfo.builder()
                                .codeCd(m.getCodeCd())
                                .codeNm(m.getCodeNm())
                                .codeDtlList(m.getCodeDtlList().stream()
                                        .map(n -> CodeDtlInfo.builder()
                                                .codeDtlCd(n.getCodeDtlCd())
                                                .codeDtlNm(n.getCodeDtlNm())
                                                .build())
                                        .collect(Collectors.toList()))
                                .build())
                        .collect(Collectors.toList()))
                .authMenuList(authMenuList.stream()
                        .map(m -> AuthMenuInfo.builder()
                                .menuNo(m.getMenuNo())
                                .topMenuNo(m.getTopMenuNo())
                                .menuId(m.getMenuId())
                                .menuNm(m.getMenuNm())
                                .topMenuNm(m.getTopMenuNm())
                                .viewNum(m.getViewNum())
                                .level(m.getLevel())
                                .useYn(m.getUseYn())
                                .useYnNm(m.getUseYnNm())
                                .openCd(m.getOpenCd())
                                .bMenuCnt(m.getBMenuCnt())
                                .connUrl(m.getConnUrl())
                                .build())
                        .collect(Collectors.toList()))
                .actList(actList.stream()
                        .map(m -> ActInfo.builder()
                                .menuNo(m.getMenuNo())
                                .actId(m.getActId())
                                .actTpCd(m.getActTpCd())
                                .menuId(m.getMenuId())
                                .actNm(m.getActNm())
                                .connUrl(m.getConnUrl())
                                .viewYn(m.getViewYn())
                                .build())
                        .collect(Collectors.toList()))
                .build();
    }

    @Getter
    @Builder
    @AllArgsConstructor(access = AccessLevel.PROTECTED)
    static class CodeInfo {
        @ApiModelProperty(value = "코드")
        private String codeCd;

        @ApiModelProperty(value = "코드명")
        private String codeNm;

        @ApiModelProperty(value = "상세코드 리스트")
        private List<CodeDtlInfo> codeDtlList;
    }

    @Getter
    @Builder
    @AllArgsConstructor(access = AccessLevel.PROTECTED)
    static class CodeDtlInfo {
        @ApiModelProperty(value = "상세코드")
        private String codeDtlCd;

        @ApiModelProperty(value = "상세코드명")
        private String codeDtlNm;
    }

    @Getter
    @Builder
    @AllArgsConstructor(access = AccessLevel.PROTECTED)
    static class AuthMenuInfo {
        @ApiModelProperty(value = "메뉴번호")
        private int menuNo;

        @ApiModelProperty(value = "상위메뉴번호")
        private int topMenuNo;

        @ApiModelProperty(value = "메뉴ID")
        private String menuId;

        @ApiModelProperty(value = "메뉴명")
        private String menuNm;

        @ApiModelProperty(value = "상위메뉴명")
        private String topMenuNm;

        @ApiModelProperty(value = "출력순번")
        private int viewNum;

        @ApiModelProperty(value = "레벨")
        private int level;

        @ApiModelProperty(value = "사용여부")
        private String useYn;

        @ApiModelProperty(value = "사용여부명")
        private String useYnNm;

        @ApiModelProperty(value = "출력구분(S-시스템, T-타이틀, C-화면전환, P-팝업")
        private String openCd;

        @ApiModelProperty(value = "하위 메뉴 카운트")
        private Integer bMenuCnt;

        @ApiModelProperty(value = "메뉴URL")
        private String connUrl;
    }



    @Getter
    @Builder
    @AllArgsConstructor(access = AccessLevel.PROTECTED)
    static class ActInfo {
        @ApiModelProperty(value = "메뉴번호")
        private Integer menuNo;

        @ApiModelProperty(value = "액션ID")
        private String actId;

        @ApiModelProperty(value = "액션명")
        private String actNm;

        @ApiModelProperty(value = "액션타입코드")
        private String actTpCd;

        @ApiModelProperty(value = "메뉴ID")
        private String menuId;

        @ApiModelProperty(value = "호출URL")
        private String connUrl;

        @ApiModelProperty(value = "출력여부")
        private String viewYn;

    }

    @Getter
    @Builder
    @AllArgsConstructor(access = AccessLevel.PROTECTED)
    static class UserInfo {
        @ApiModelProperty(value = "사용자아이디")
        private String userId;

        @ApiModelProperty(value = "사용자명")
        private String userNm;

        @ApiModelProperty(value = "사용자권한코드")
        private String authTpCd;

        @ApiModelProperty(value = "사용자권한명")
        private String authTpNm;

        @ApiModelProperty(value = "브랜드그룹아이디")
        private String afflShopId;

        @ApiModelProperty(value = "브랜드그룹명")
        private String afflShopNm;

        @ApiModelProperty(value = "브랜드아이디")
        private String brandId;

        @ApiModelProperty(value = "브랜드명")
        private String brandNm;

        @ApiModelProperty(value = "매장아이디")
        private String storeId;

        @ApiModelProperty(value = "매장명")
        private String storeNm;

        @ApiModelProperty(value = "공급사아이디")
        private String spplyId;

        @ApiModelProperty(value = "공급사명")
        private String spplyNm;

        @ApiModelProperty(value = "시스템관리자여부")
        private String setSystemAdminYn;

        @ApiModelProperty(value = "오더퀸 브랜드코드")
        private String oqBrandId;

        @ApiModelProperty(value = "오더퀸 매장코드")
        private String oqStoreId;

        @ApiModelProperty(value = "카드결재가능여부")
        private String apprYn;

        @ApiModelProperty(value = "카드결재MallId")
        private String mallId;

        @ApiModelProperty(value = "간소화여부 1 간소화 , 0 절차")
        private String shmtColProcType;

    }

}
